#!/usr/bin/env python
"""
Parse a .vcf format and produce genotypes to be added to a genotype network
"""
import logging
import optparse
import os
import gzip
import cPickle
import numpy
import tempfile
from custom_exceptions import *

class DataProcessingError(Exception):
    pass

def get_options():
    parser = optparse.OptionParser(usage="python {0} -f genotypes.vcf".format(__file__), description=__doc__)
    parser.add_option('-g', '-f', '--genotypes', dest='genotypes_file', 
            help='File containing the genotypes in vcf format', default=False)
    parser.add_option('-i', '--individuals', dest='individuals_file', 
            help='File containing the list of individuals and populations', default="data/individuals/individuals_annotations.txt")
    parser.add_option('-d', '--hdf5', '--h5',
            help='Use HDF5 internal storage. More memory efficient, but requires the h5py library to be installed.',
            dest='use_hdf5', action='store_true', default=False)
    parser.add_option('-u', '--debug', dest='debug', action='store_true', default=True)
    (options, args) = parser.parse_args()
    
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, filename='logs/vcf_to_binary_convert.log')

    if (options.genotypes_file is False) and (len(args) == 0):
        parser.print_help()
        parser.error('parse_genotypes.py error: no genotypes file provided')
    try:
        genotypes_file_path = ''
        if options.genotypes_file is not False:
            genotypes_file_path = options.genotypes_file
        elif args != '':
            genotypes_file_path = args[0]
        if os.stat(genotypes_file_path).st_size == 0:
            parser.error("file {0} is empty".format(genotypes_file_path))
    except:
        print __doc__
        parser.error("Can not open genotypes file {0}".format(options.genotypes_file))

    return (genotypes_file_path, options)

def read_genotypes_vcf_h5(vcf_file_path, individuals_dict=False, gene_name=False):
    """
    Parse genotypes file in vcf format. Store intermediate variables into an HDF5 file, saving memory.

    Example vcf format:

    ::

        14      23033832        rs186383615     T       C       100     PASS    LDAF=0.0016;THETA=0.0004;AA=T;AN=2184;VT=SNP;SNPSOURCE=LOWCOV;ERATE=0.0003;RSQ=0.8538;AVGPOST=0.9995;AC=3;AF=0.0014;AFR_AF=0.0020;EUR_AF=0.0026 GT:DS:GL        0|0:0.000:-0.01,-1.48,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-3.36,-5.00     0|0:0.000:-0.00,-3.19,-5.00     0|0:0.000:-0.00,-4.22,-5.00
        14      23033838        rs1051189       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     0|0:0.000:-0.00,-3.19,-5.00     
    
    How to parse the vcf format:

    * genotypes are in the column after the 10th, one column per individual
    * 0|0 means that the individual is homozygote for the reference allele. The rest of the genotype line are quality scores etc..
    * 0/0 means that the genotype is unphased, so it should be discarded
    * as another example, 0|1 means that the first chromosome has the ref allele, and the second has the alternative allele.
    

    ===================
    Example and doctest
    ===================


    >>> from tempfile import NamedTemporaryFile
    >>> vcf_file = NamedTemporaryFile(delete=False)
    >>> sample_vcf = r'''#
    ... #CHROM  POS     ID      REF     ALT     QUAL    FILTER  INFO    FORMAT  HG00096 HG00097 HG00099 HG00100 HG00101
    ... 14      23033832        rs186383615     T       C       100     PASS    LDAF=0.0016;THETA=0.0004;AA=T;AN=2184;VT=SNP;SNPSOURCE=LOWCOV;ERATE=0.0003;RSQ=0.8538;AVGPOST=0.9995;AC=3;AF=0.0014;AFR_AF=0.0020;EUR_AF=0.0026 GT:DS:GL        1|0:0.000:-0.01,-1.48,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-3.36,-5.00     0|1:0.000:-0.00,-3.19,-5.00     0|0:0.000:-0.00,-4.22,-5.00
    ... 14      23033840        rs1051189       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|1:0.000:-0.00,-2.31,-5.00     0|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... 14      23033854        rs1051190       C       G       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     0|1:0.000:-0.00,-3.19,-5.00     1|1:0.000:-0.00,-2.31,-5.00
    ... 14      23033888        rs1051191       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     1|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... '''
    >>> vcf_file.write(sample_vcf)
    >>> vcf_file.close()
    >>> (indv_annotations, genotypes, unphased_genotypes, snp_ids, snp_annotations) = read_genotypes_vcf_h5(vcf_file.name)
    >>> print ' '.join(snp_annotations['pos'])
    23033832 23033840 23033854 23033888

    >>> for ind_id in range(len(indv_annotations)): 
    ...     print indv_annotations[ind_id, 0], indv_annotations[ind_id, 1], indv_annotations[ind_id, 2], indv_annotations[ind_id, 3], indv_annotations[ind_id, 4],
    ...     for geno_value in genotypes[ind_id, :]:
    ...         print geno_value,
    ...     print #doctest: +ELLIPSIS
    tmp... HG00096_1 nopop nocontinent X 1 0 0 0
    tmp... HG00096_2 nopop nocontinent X 0 0 0 0
    tmp... HG00097_1 nopop nocontinent X 0 0 1 1
    tmp... HG00097_2 nopop nocontinent X 0 0 0 0
    tmp... HG00099_1 nopop nocontinent X 0 0 0 0
    tmp... HG00099_2 nopop nocontinent X 0 1 0 0
    tmp... HG00100_1 nopop nocontinent X 0 0 0 1
    tmp... HG00100_2 nopop nocontinent X 1 0 1 0
    tmp... HG00101_1 nopop nocontinent X 0 1 1 1
    tmp... HG00101_2 nopop nocontinent X 0 0 1 0

    """
    logging.debug( "parsing " + vcf_file_path)
    import h5py

    if vcf_file_path[-3:] == '.gz':
        vcf_file = gzip.open(vcf_file_path)
    else: 
        vcf_file = open(vcf_file_path, 'r')
    
    if gene_name is False:
        h5_file = tempfile.NamedTemporaryFile()
        gene_name = h5_file.name.split('/')[-1]
    else:
        h5_file = open('./data/hdf5/' + gene_name + '_genotypes.hdf5', 'w')
    h5_tmp_file = h5py.File(h5_file.name, 'w')

    snp_ids = []
    snp_annotations = {"chrom": [], "pos": [], "ref":[]}
    snp_number = 0
    current_numbercolumns = 100
    for snp_line in vcf_file:
        if snp_line.strip() == '':
            continue

        # READ HEADERS - FIRST LINE
        if snp_line.startswith('#CHROM'):
            snp_fields = snp_line.split()
            snp_id = snp_fields[2]
            individuals_names = snp_fields[9:]

            n_individuals = len(individuals_names)
            my_dtypes = numpy.dtype('S10')#, 'S10', 'S10', 'S2')
            # genotype_line is a matrix containing one individual per row, and one genotype per column.
            genotype_line = h5_tmp_file.create_dataset("genotype_line", (n_individuals*2, current_numbercolumns), dtype='b', maxshape=(n_individuals*2, None))
            indv_annotations = h5_tmp_file.create_dataset("indv_annotations", (n_individuals*2, 5), dtype='|S15')
            indv_annotations[:, 0] = gene_name

            if individuals_dict is not False:
                # if the headers contain indv_annotations on individuals, use and include them in the output file.
                individuals_pops = [individuals_dict[name]['subpop'] for name in individuals_names]
                individuals_continents = [individuals_dict[name]['continent'] for name in individuals_names]

                indv_annotations[0::2, 1] = [name + '_1' for name in individuals_names]
                indv_annotations[1::2, 1] = [name + '_2' for name in individuals_names]

                indv_annotations[0::2, 2] = individuals_pops
                indv_annotations[1::2, 2] = individuals_pops

                indv_annotations[0::2, 3] = individuals_continents
                indv_annotations[1::2, 3] = individuals_continents

                indv_annotations[:, 4] = 'X'
            else:
                indv_annotations[0::2, 1] = [name + '_1' for name in individuals_names]
                indv_annotations[1::2, 1] = [name + '_2' for name in individuals_names]

                indv_annotations[:, 2] = "nopop"
                indv_annotations[:, 3] = "nocontinent"
                indv_annotations[:, 4] = "X"

#            unphased_genotypes      = [[name + '_unphased'] for name in individuals_names]
#            print indv_annotations[:,:]
            
        # READ DATA LINES
        if not snp_line.startswith('#'):
            print line
            snp_fields = snp_line.split(sep=None)
            (chromosome, position, snp_id, ref, alt, qual, filt, info, geno_format) = snp_fields[:9]
            snp_annotations["chrom"].append(snp_fields[0])
            snp_annotations["pos"].append(snp_fields[1])
            snp_annotations["ref"].append(snp_fields[3])

            snp_ids.append(snp_id)
            chromosome1          = [int(snp_fields[i][0]) for i in range(9, len(snp_fields))]
#            print len(chromosome1)
            chromosome2          = [int(snp_fields[i][2]) for i in range(9, len(snp_fields))]
            unphased_individuals = [snp_fields[i][1] for i in range(9, len(snp_fields))]
            if unphased_individuals.count('/') > 0:
                raise GenotypeNetworkInitError("unphased genotypes in {0}".format(snp_id))
#                logging.warn("unphased genotypes in {0}".format(snp_id))
            if chromosome1.count('2') > 0 or chromosome2.count('2') > 0:
                raise GenotypeNetworkInitError("triallelic SNP in {0}".format(snp_id))
                logging.warn("triallelic SNP in {0}".format(snp_id))

#            print chromosome1
            # TODO: filter out unphased individuals

            # Updating the number of columns in the genotype_line HDF5 array.
            if snp_number >= current_numbercolumns:
                current_numbercolumns += 100
                genotype_line.resize(size=current_numbercolumns, axis=1)
            genotype_line[::2,snp_number] = chromosome1
            genotype_line[1::2,snp_number] = chromosome2
            snp_number += 1

#            [unphased_genotypes[i].append(unphased_individuals[i]) for i in range(len(unphased_individuals))]
    genotype_line.resize(size=snp_number, axis=1) 

    snp_annotations_h5 = h5_tmp_file.create_dataset("snp_annotations", (snp_number, 4), "S15")
    snp_annotations_h5[:,0] = snp_ids
    snp_annotations_h5[:,1] = snp_annotations["chrom"]
    snp_annotations_h5[:,2] = snp_annotations["pos"]
    snp_annotations_h5[:,3] = snp_annotations["ref"]

    # TODO: filter out unphased individuals
#    genotype_lines = [genotype_line_1, genotype_line_2]
#    genotype_lines.extend(genotype_line_2)
    unphased_genotypes = []
    return((indv_annotations, genotype_line, unphased_genotypes, snp_ids, snp_annotations))


def read_genotypes_vcf(vcf_file_path, individuals_dict=False, individuals_file=""): # NOTE: the individuals_file here is passed only for improving the error message
    """
    Parse genotypes file in vcf format

    Example vcf format:

    ::

        14      23033832        rs186383615     T       C       100     PASS    LDAF=0.0016;THETA=0.0004;AA=T;AN=2184;VT=SNP;SNPSOURCE=LOWCOV;ERATE=0.0003;RSQ=0.8538;AVGPOST=0.9995;AC=3;AF=0.0014;AFR_AF=0.0020;EUR_AF=0.0026 GT:DS:GL        0|0:0.000:-0.01,-1.48,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-3.36,-5.00     0|0:0.000:-0.00,-3.19,-5.00     0|0:0.000:-0.00,-4.22,-5.00
        14      23033838        rs1051189       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     0|0:0.000:-0.00,-3.19,-5.00     
    
    How to parse the vcf format:

    * genotypes are in the column after the 10th, one column per individual
    * 0|0 means that the individual is homozygote for the reference allele. The rest of the genotype line are quality scores etc..
    * 0/0 means that the genotype is unphased, so it should be discarded
    * as another example, 0|1 means that the first chromosome has the ref allele, and the second has the alternative allele.
    

    ===================
    Example and doctest
    ===================


    >>> from tempfile import NamedTemporaryFile
    >>> vcf_file = NamedTemporaryFile(delete=False)
    >>> sample_vcf = r'''#
    ... #CHROM  POS     ID      REF     ALT     QUAL    FILTER  INFO    FORMAT  HG00096 HG00097 HG00099 HG00100 HG00101
    ... 14      23033832        rs186383615     T       C       100     PASS    LDAF=0.0016;THETA=0.0004;AA=T;AN=2184;VT=SNP;SNPSOURCE=LOWCOV;ERATE=0.0003;RSQ=0.8538;AVGPOST=0.9995;AC=3;AF=0.0014;AFR_AF=0.0020;EUR_AF=0.0026 GT:DS:GL        1|0:0.000:-0.01,-1.48,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-3.36,-5.00     0|1:0.000:-0.00,-3.19,-5.00     0|0:0.000:-0.00,-4.22,-5.00
    ... 14      23033840        rs1051189       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|1:0.000:-0.00,-2.31,-5.00     0|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... 14      23033854        rs1051190       C       G       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     0|1:0.000:-0.00,-3.19,-5.00     1|1:0.000:-0.00,-2.31,-5.00
    ... 14      23033888        rs1051191       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     1|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... '''
    >>> vcf_file.write(sample_vcf)
    >>> vcf_file.close()
    >>> (genotype_lines, unphased_genotypes, snp_ids, snp_annotations) = read_genotypes_vcf(vcf_file.name)
    >>> for genotype in genotype_lines: 
    ...     print(' '.join(genotype)) #doctest: +ELLIPSIS
    HG00096_1 nopop nocontinent X 1 0 0 0
    HG00097_1 nopop nocontinent X 0 0 1 1
    HG00099_1 nopop nocontinent X 0 0 0 0
    HG00100_1 nopop nocontinent X 0 0 0 1
    HG00101_1 nopop nocontinent X 0 1 1 1
    HG00096_2 nopop nocontinent X 0 0 0 0
    HG00097_2 nopop nocontinent X 0 0 0 0
    HG00099_2 nopop nocontinent X 0 1 0 0
    HG00100_2 nopop nocontinent X 1 0 1 0
    HG00101_2 nopop nocontinent X 0 0 1 0

    >>> print ' '.join(snp_annotations['pos'])
    23033832 23033840 23033854 23033888

    """
    logging.debug( vcf_file_path)

    if vcf_file_path[-3:] == '.gz':
        vcf_file = gzip.open(vcf_file_path)
    else: 
        vcf_file = open(vcf_file_path, 'r')

    snp_ids = []
    snp_annotations = {"chrom": [], "pos": [], "ref":[]}
#    print individuals_dict
    for snp_line in vcf_file:
#            logging.debug( ("snp line", snp_line[:20]))
        if snp_line.strip() == '':
            continue

        # READ HEADERS - FIRST LINE
        if snp_line.startswith('#CHROM'):
            snp_fields = snp_line.split()
#                individuals = individuals_names.fromkeys(snp_fields[9:], [])
            snp_id = snp_fields[2]
            individuals_names = snp_fields[9:]
            if individuals_dict is not False:
                for name in individuals_names:
                    if not individuals_dict.has_key(name):
                        raise InvalidIndividualsFile("The individual {0} is not defined in the individuals_file".format(name), individuals_file)
                individuals_pops = [individuals_dict[name]['subpop'] for name in individuals_names]
                individuals_continents = [individuals_dict[name]['continent'] for name in individuals_names]
                individuals_sex = ['X' for name in individuals_names]
                genotype_line_1 = [[name + '_1', individuals_dict[name]['subpop'], individuals_dict[name]['continent'], 'X']        for name in individuals_names]
                genotype_line_2 = [[name + '_2', individuals_dict[name]['subpop'], individuals_dict[name]['continent'], 'X']        for name in individuals_names]
            else:
                # This should be a fallback function in case the individuals_dict is missing, as in the simulations. #TODO: determine if this is really needed now.
                individuals_pops = ['no_pop' for name in individuals_names]
                individuals_continents = ['no_continent' for name in individuals_names]
                individuals_sex = ['X' for name in individuals_names]
                genotype_line_1 = [[name + '_1', 'nopop', 'nocontinent', 'X']        for name in individuals_names]
                genotype_line_2 = [[name + '_2', 'nopop', 'nocontinent', 'X']        for name in individuals_names]
#            genotype_line_1 = [[name + '_1']        for name in individuals_names]
#            genotype_line_2 = [[name + '_2']        for name in individuals_names]
#            unphased_genotypes      = [[name + '_unphased'] for name in individuals_names]
            
        # READ DATA LINES
        if not snp_line.startswith('#'):
            snp_fields = snp_line.split()
            if not len(snp_fields) > 8:
                raise DataProcessingError("""
                Wrong number of columns in the VCF file. 
                
                Check that:
                    - you are providing a correct VCF file with the -f (or -g) option, 
                    - there are no empty lines in the file
                    - all the comment lines begin with '#'
                    - each genotype line contains the same number of columns, at least 8

                Last line parsed: 
                 
            """ + snp_line)

            (chromosome, position, snp_id, ref, alt, qual, filt, info, geno_format) = snp_fields[:9]
            snp_annotations["chrom"].append(snp_fields[0])
            snp_annotations["pos"].append(snp_fields[1])
            snp_annotations["ref"].append(snp_fields[3])

            snp_ids.append(snp_id)
            chromosome1          = [snp_fields[i][0] for i in range(9, len(snp_fields))]
            chromosome2          = [snp_fields[i][2] for i in range(9, len(snp_fields))]
            unphased_individuals = [snp_fields[i][1] for i in range(9, len(snp_fields))]
            if unphased_individuals.count('/') > 0:
#                print snp_line
                raise GenotypeNetworkInitError("""
    The input file {0} contains an unphased Genotype.
    
    vcf2networks does not support unphased genotypes. 
    In vcf2networks, the two haplotypes of each individual are threated as separate entities; so, in case on unphased genotypes, there is no way to know to which haplotype a genotype would belong.
    You should filter your vcf file and remove all the unphased genotypes. For example:

        $: vcftools --phased --vcf {0} --recode --out {0}.filtered

        see http://vcftools.sourceforge.net for more options.

    The SNP containing the unphased genotype is: {1}. 

    Unphased genotypes can be distinguished from phased genotypes because the two genotypes are separated by a "/" instead of a "|".
    E.g.:
        0|0:0.000:-0.05,-0.94,-5.00 -> phased genotypes (0 and 0)
        0/0:0.000:-0.05,-0.94,-5.00 -> unphased genotypes (0 and 0)

    """.format(vcf_file.name, snp_id))


            if chromosome1.count('2') > 0 or chromosome2.count('2') > 0:
                raise GenotypeNetworkInitError("""
    The input file {0} contains a triallelic SNP.
    
    vcf2networks does not currently support triallelic SNPs, and does not support any other type of variation except for SNPs.
    You should filter your vcf file and remove all the genotypes which are not biallelic:
    
        $: vcftools --min-alleles 2 --max-alleles 2 --vcf {0} --recode --out {0}.filtered
        
        see http://vcftools.sourceforge.net for more options.

    The SNP containing the triallelic SNP is: {1}. 

    Triallelic genotypes can be distinguished because the "ALT" column contain more than one allele, and they contain a number different from 0 or 1 in their genotype columns:
    E.g.:
        2|0:0.000:-0.05,-0.94,-5.00 -> Triallelic SNP (the first chromosome has the allele "2")

        """.format(vcf_file.name, snp_id))

            # TODO: filter out unphased individuals
            
            [genotype_line_1[i].append(           chromosome1[i]) for i in range(len(chromosome1))]
            [genotype_line_2[i].append(           chromosome2[i]) for i in range(len(chromosome2))]
#            [     unphased_genotypes[i].append(  unphased_individuals[i]) for i in range(len(unphased_individuals))]

    # TODO: filter out unphased individuals
    genotype_lines = genotype_line_1
    genotype_lines.extend(genotype_line_2)
    unphased_genotypes = []
    return((genotype_lines, unphased_genotypes, snp_ids, snp_annotations))



def parse_individuals_file(individuals_file_path="./data/individuals/individuals_annotations.txt",
                                pickle_from_file_if_possible=False,
                                picklefile='./data/pickles/individuals_dict.pickle'):
    """
    Parse the file containing info on individuals and populations, 
    return a dictionary of individuals and annotations.

    ++++++++++++++++++++++++++++
    Example of individuals file:
    ++++++++++++++++++++++++++++

    #ID POP CONTINENT   PHENOTYPE1
    HG00096 GBR     EUR     happy
    HG00097 GBR     EUR     happy
    HG00099 GBR     EUR     sad
    HG00100 GBR     EUR     sad
    HG00101 GBR     EUR     happy
    HG00102 GBR     EUR     happy
    HG00103 GBR     EUR     sad
    HG00104 GBR     EUR     happy
    HG00106 GBR     EUR     happy
    HG00108 GBR     EUR     sad
    HG00109 GBR     EUR     happy
    HG00110 GBR     EUR     happy

    >>> individuals_dict = parse_individuals_file()
    >>> print individuals_dict["HG01083"] == {'continent': 'AMR', 'subpop': 'PUR', 'phenotype1': ['ILLUMINA']}
    True
    >>> print individuals_dict['HG00113'] == {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': ['ABI_SOLID', 'ILLUMINA']}
    True

    """
    try:
        assert pickle_from_file_if_possible  # if the option pickle_from_file_if_possible is false, this block will return an error
        individuals_dict = cPickle.load(open(picklefile, 'r'))
#        individuals_dict['HG00096']
    except:
        individuals_dict = {}
        with open(individuals_file_path, 'r') as individuals_file:
            header = individuals_file.readline()
            if not header.startswith('#'):
                raise InvalidIndividualsFile(
                        'The first line in the individuals file must begin with a "#", and contain the names of the columns encoded in the file',
                        individuals_file_path)
            header_fields = header.replace('#', '').split()
            number_columns = len(header_fields)
#            print header_fields
#            print len(header_fields) <3
            if number_columns < 3:
                raise InvalidIndividualsFile(
                        'The individuals file should contain at least three columns, separated by tabulations.',
                        individuals_file_path)
            if header_fields[0].lower() not in ("id", "name", "individual"):
                raise InvalidIndividualsFile(
                        'The first column of the individuals file should be named "ID", or "individual", or "name"',
                        individuals_file_path)
            if header_fields[1].lower() not in ("pop", "population", "subpop", "group"):
                raise InvalidIndividualsFile(
                        'The second column of the individuals file should be named "POP", and describe the population of the individuals.\n\tIn future releases, this column will not be mandatory, but in the current version your individuals_file must contain a column named "POP" in the second position.\n\tSorry for the inconvenience.',
                        individuals_file_path)
            if header_fields[2].lower() not in ("continent", "cont", "continental_group"):
                raise InvalidIndividualsFile(
                        'The third column of the individuals file should be named "CONTINENT", and describe the continental group to which the individuals belong.\n\tIn future releases, this column will not be mandatory, but in the current version your individuals_file must contain a column named "CONTINENT" in the third position.\n\tSorry for the inconvenience.',
                        individuals_file_path)


            # Parsing the other rows
            for line in individuals_file:
                if not line.strip() == '':
                    fields = line.split()
                    if len(fields) != number_columns:
                        raise InvalidIndividualsFile(
                                """
    One line in your individuals file contains a different number of columns than expected.
    The line is:
    
        {0}
        
    It should have exactly {1} columns, defined as {2}""".format(line, number_columns, ', '.join(header_fields)), individuals_file)
                    ind = fields[0]
                    subpop = fields[1]
                    continent = fields[2]
#                    techniques = [x.replace(',','') for x in fields[3:]]
                    individuals_dict[ind] = {'subpop': subpop, 'continent': continent} 
                    for phenotype_id in range(len(header_fields[3:])):
#                        print header_fields[phenotype_id+3], fields[phenotype_id+3]
                        individuals_dict[ind][header_fields[phenotype_id+3]] = fields[phenotype_id+3]

        if pickle_from_file_if_possible is True:
            cPickle.dump(individuals_dict, open(picklefile, 'w'))
    return individuals_dict


def print_binary_file(genotypes_file_path, options):
    """
    Print the output of binary file conversion to screen.

    >>> from tempfile import NamedTemporaryFile
    >>> import optparse
    >>> options = optparse.OptionParser()
    >>> options.use_hdf5 = False
    >>> options.debug = False
    >>> options.individuals_file = "data/individuals/individuals_annotations.txt"
    >>> vcf_file = NamedTemporaryFile(delete=False)
    >>> sample_vcf = r'''#
    ... #CHROM  POS     ID      REF     ALT     QUAL    FILTER  INFO    FORMAT  HG00096 HG00097 HG00099 HG00100 HG00101
    ... 14      23033832        rs186383615     T       C       100     PASS    LDAF=0.0016;THETA=0.0004;AA=T;AN=2184;VT=SNP;SNPSOURCE=LOWCOV;ERATE=0.0003;RSQ=0.8538;AVGPOST=0.9995;AC=3;AF=0.0014;AFR_AF=0.0020;EUR_AF=0.0026 GT:DS:GL        1|0:0.000:-0.01,-1.48,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-3.36,-5.00     0|1:0.000:-0.00,-3.19,-5.00     0|0:0.000:-0.00,-4.22,-5.00
    ... 14      23033840        rs1051189       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     0|0:0.000:0.00,-5.00,-5.00      0|1:0.000:-0.00,-2.31,-5.00     0|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... 14      23033854        rs1051190       C       G       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     0|1:0.000:-0.00,-3.19,-5.00     1|1:0.000:-0.00,-2.31,-5.00
    ... 14      23033888        rs1051191       G       A       100     PASS    RSQ=0.9958;AC=248;THETA=0.0004;AA=G;AN=2184;AVGPOST=0.9990;VT=SNP;LDAF=0.1138;SNPSOURCE=LOWCOV;ERATE=0.0003;AF=0.11;ASN_AF=0.15;AMR_AF=0.07;AFR_AF=0.13;EUR_AF=0.10     GT:DS:GL        0|0:0.000:-0.01,-1.77,-5.00     1|0:0.000:0.00,-5.00,-5.00      0|0:0.000:-0.00,-2.31,-5.00     1|0:0.000:-0.00,-3.19,-5.00     1|0:0.000:-0.00,-2.31,-5.00
    ... '''
    >>> vcf_file.write(sample_vcf)
    >>> vcf_file.close()
    >>> print(print_binary_file(vcf_file.name, options)) # doctest: +ELLIPSIS
    #gene chromosome_id snps
    #SNP IDS: rs186383615 rs1051189 rs1051190 rs1051191
    #Chromosome: 14 14 14 14
    #Position: 23033832 23033840 23033854 23033888
    #Ref. allele: T G C G
    tmp... HG00096_1 GBR EUR X 1 0 0 0
    tmp... HG00097_1 GBR EUR X 0 0 1 1
    tmp... HG00099_1 GBR EUR X 0 0 0 0
    tmp... HG00100_1 GBR EUR X 0 0 0 1
    tmp... HG00101_1 GBR EUR X 0 1 1 1
    tmp... HG00096_2 GBR EUR X 0 0 0 0
    tmp... HG00097_2 GBR EUR X 0 0 0 0
    tmp... HG00099_2 GBR EUR X 0 1 0 0
    tmp... HG00100_2 GBR EUR X 1 0 1 0
    tmp... HG00101_2 GBR EUR X 0 0 1 0


    This script can also use an internal HDF5 file to store genotypes. This 
    requires the h5py and HDF5 libraries to be installed, but is more memory
    efficient, as genotypes are stored on disk instead of RAM memory.
    When using this option, the output format is slighly different, as the order
    of individuals is more correct. However, the results are the same.
    >>> options.use_hdf5 = True
    >>> print_binary_file(vcf_file.name, options) # doctest: +ELLIPSIS
    #gene chromosome_id snps
    #SNP IDS: rs186383615 rs1051189 rs1051190 rs1051191
    #Chromosome: 14 14 14 14
    #Position: 23033832 23033840 23033854 23033888
    #Ref. allele: T G C G
    tmp... HG00096_1 GBR EUR X 1 0 0 0
    tmp... HG00096_2 GBR EUR X 0 0 0 0
    tmp... HG00097_1 GBR EUR X 0 0 1 1
    tmp... HG00097_2 GBR EUR X 0 0 0 0
    tmp... HG00099_1 GBR EUR X 0 0 0 0
    tmp... HG00099_2 GBR EUR X 0 1 0 0
    tmp... HG00100_1 GBR EUR X 0 0 0 1
    tmp... HG00100_2 GBR EUR X 1 0 1 0
    tmp... HG00101_1 GBR EUR X 0 1 1 1
    tmp... HG00101_2 GBR EUR X 0 0 1 0

    >>> vcf_file.unlink(vcf_file.name)

    """
    gene_name = genotypes_file_path.split('/')[-1].split('.',1)[0]
#    print options.individuals_file
    individuals_dict = parse_individuals_file(options.individuals_file)
    if options.use_hdf5 is True:
        (indv_annotations, genotypes, unphased_genotypes, snp_ids, snp_annotations) = read_genotypes_vcf_h5(genotypes_file_path, individuals_dict, gene_name)
    else:   
        (genotypes, unphased_genotypes, snp_ids, snp_annotations) = read_genotypes_vcf(genotypes_file_path, individuals_dict, options.individuals_file)
    output = '#gene chromosome_id snps'
    output += '\n#SNP IDS: ' + ' '.join(snp_ids)
    output += '\n#Chromosome: ' + ' '.join(snp_annotations['chrom'])
    output += '\n#Position: ' + ' '.join(snp_annotations['pos'])
    output += '\n#Ref. allele: ' + ' '.join(snp_annotations['ref'])
    if options.use_hdf5 is True:
        for ind_id in range(len(indv_annotations)):
            output += indv_annotations[ind_id, 0], indv_annotations[ind_id, 1], indv_annotations[ind_id, 2], indv_annotations[ind_id, 3], indv_annotations[ind_id, 4] + ' '
            for geno_value in genotypes[ind_id, :]:
                output += geno_value + ''
            output += ''
    else:
        for genotype in genotypes:
            output += '\n' + gene_name + ' ' + ' '.join(genotype)
    return output

if __name__ == '__main__':
    (genotypes_file_path, options) = get_options()
    print_binary_file(genotypes_file_path, options)
    logging.debug("Finished parsing file {0}".format(genotypes_file_path))




#!/usr/bin/env python
"""
"""
#from test_SubgraphByPop import SimpleNetwork
import cosi_to_binary
import unittest
import nose
import subprocess


def test_CosiToBinaryDoctest():
    """
    >>> from cosi_to_binary import read_hap_file
    >>> from tempfile import NamedTemporaryFile
    >>> hapfile = NamedTemporaryFile(delete=False)
    >>> hapfile.write(r'''
    ... 0       1       1 1 2 2 2 1 2 1
    ... 1       1       2 1 2 2 2 1 1 1
    ... 2       1       2 1 2 2 1 2 2 2
    ... 3       1       1 1 2 2 2 1 2 1
    ... 4       1       2 2 1 1 1 1 1 2
    ... 5       1       2 1 1 2 2 1 2 1
    ... 6       1       1 2 2 2 2 1 2 1
    ... 7       1       1 1 1 2 1 1 2 1
    ... ''')

    >>> hapfile.close()

    ===========================
    Example Output binary file
    ===========================
    >>> print (read_hap_file(hapfile.name)) #doctest: +NORMALIZE_WHITESPACE
    #binary file v2. Simulations results. MAF filter: 0.01. Only SNPs that have MAF > 0.01 in each population are kept.
    #number of SNPs: 8. SNPs filtered: 0
    neutral_sim hap0_pop1 pop1 EUR X 0 0 1 1 1 0 1 0
    neutral_sim hap1_pop1 pop1 EUR X 1 0 1 1 1 0 0 0
    neutral_sim hap2_pop1 pop1 EUR X 1 0 1 1 0 1 1 1
    neutral_sim hap3_pop1 pop1 EUR X 0 0 1 1 1 0 1 0
    neutral_sim hap4_pop1 pop1 EUR X 1 1 0 0 0 0 0 1
    neutral_sim hap5_pop1 pop1 EUR X 1 0 0 1 1 0 1 0
    neutral_sim hap6_pop1 pop1 EUR X 0 1 1 1 1 0 1 0
    neutral_sim hap7_pop1 pop1 EUR X 0 0 0 1 0 0 1 0
    <BLANKLINE>

    """

def test_GlobalPopFile():
    """
    parse a COSI file containing 3 populations.
    
    >>> from cosi_to_binary import read_hap_file
    >>> from tempfile import NamedTemporaryFile
    >>> hapfile2 = NamedTemporaryFile(prefix='hap2', delete=False)
    >>> hapfile2.write(r'''0       1       2 1 2 2 2 
    ... 1       1       2 1 1 1 2
    ... 2       1       2 1 2 2 1
    ... 3       2       2 1 1 1 1
    ... 4       2       2 1 2 2 2
    ... 5       2       2 1 1 1 1
    ... 6       3       2 1 1 2 2
    ... 7       3       2 1 2 1 1
    ... 8       3       2 1 1 1 2
    ... ''')
    >>> hapfile2.close()


    >>> print (read_hap_file(hapfile2.name))  #doctest: +NORMALIZE_WHITESPACE
    #binary file v2. Simulations results. MAF filter: 0.01. Only SNPs that have MAF > 0.01 in each population are kept.
    #number of SNPs: 5. SNPs filtered: 2
    neutral_sim hap0_pop1 pop1 EUR X 1 1 1
    neutral_sim hap1_pop1 pop1 EUR X 0 0 1
    neutral_sim hap2_pop1 pop1 EUR X 1 1 0
    neutral_sim hap0_pop3 pop3 AFR X 0 1 1
    neutral_sim hap1_pop3 pop3 AFR X 1 0 0
    neutral_sim hap2_pop3 pop3 AFR X 0 0 1
    neutral_sim hap0_pop2 pop2 ASN X 0 0 0
    neutral_sim hap1_pop2 pop2 ASN X 1 1 1
    neutral_sim hap2_pop2 pop2 ASN X 0 0 0
    <BLANKLINE>

    >>> hapfile2.unlink(hapfile2.name)
    """

def test_GlobalPopFileAssymetric():
    """
    parse a COSI file containing 3 populations, where one population contains an homorphic SNP.
    
    >>> from cosi_to_binary import read_hap_file
    >>> from tempfile import NamedTemporaryFile
    >>> hapfile2 = NamedTemporaryFile(prefix='hap2', delete=False)
    >>> hapfile2.write(r'''0       1       2 1 2 2 2 
    ... 1       1       2 1 1 1 2
    ... 2       1       2 1 2 2 1
    ... 3       2       2 1 1 1 1
    ... 4       2       2 1 2 2 2
    ... 5       2       2 1 1 1 1
    ... 6       3       2 1 1 2 2
    ... 7       3       2 1 2 1 2
    ... 8       3       2 1 1 1 2
    ... ''')
    >>> hapfile2.close()


    >>> print (read_hap_file(hapfile2.name))  #doctest: +NORMALIZE_WHITESPACE
    #binary file v2. Simulations results. MAF filter: 0.01. Only SNPs that have MAF > 0.01 in each population are kept.
    #number of SNPs: 5. SNPs filtered: 3
    neutral_sim hap0_pop1 pop1 EUR X 1 1
    neutral_sim hap1_pop1 pop1 EUR X 0 0
    neutral_sim hap2_pop1 pop1 EUR X 1 1
    neutral_sim hap0_pop3 pop3 AFR X 0 1
    neutral_sim hap1_pop3 pop3 AFR X 1 0
    neutral_sim hap2_pop3 pop3 AFR X 0 0
    neutral_sim hap0_pop2 pop2 ASN X 0 0
    neutral_sim hap1_pop2 pop2 ASN X 1 1
    neutral_sim hap2_pop2 pop2 ASN X 0 0
    <BLANKLINE>

    >>> hapfile2.unlink(hapfile2.name)
    """




class ShortCosiSimulation(unittest.TestCase):
    hapfile = 'test/test_data/simulations/cosiout_short.hap-1'

    # number of snps == number of rows the input file, minus those filtered by MAF
    expected_number_snps = 30

    # number of chromosomes == number of columns in input file
    # in this case, the short cosi sim file has been generating by taking the first 49 columns
    expected_number_chromosomes = 120

    def setUp(self):
        self.binary_output = cosi_to_binary.read_hap_file(self.hapfile, maf_filter=0.01)
        self.binaryoutput_lines = self.binary_output.split('\n')

    def test_NumberSnps(self):
#        print self.binaryoutput_lines[0].split()[2:]
        n_snps = len(self.binaryoutput_lines[0].split()[5:])
#        assert False
#        n_snps = len(self.binaryoutput_lines[0][17:]) / 2
        nose.tools.assert_equal(n_snps, self.expected_number_snps)
    
    def test_NumberChromosomes(self):
        n_chromosomes = len([line for line in self.binaryoutput_lines if line.startswith('neutral_sim')])
        nose.tools.assert_equal(n_chromosomes, self.expected_number_chromosomes)



#class MonomorphicCosiSimulation(ShortCosiSimulation):
#    """
#    A cosi simulation where all the SNPs are monomorphic.
#    """


@nose.plugins.attrib.attr('slow')
class LongCosiSimulation(ShortCosiSimulation):
    """
    like ShortCosiSimulation, but takes longer
    """
    hapfile = 'test/test_data/simulations/cosiout_long.hap-1'

    # number of snps == number of rows the input file, minus those filtered by MAF
    expected_number_snps = 2803

    # number of chromosomes == number of columns in input file
    expected_number_chromosomes = 120

@nose.plugins.attrib.attr('slow')
class GzippedCosiSimulation(ShortCosiSimulation):
    """
    like ShortCosiSimulation, but takes longer
    """
    hapfile = 'test/test_data/simulations/bestfit_neutral_replica_1.hap-1.gz'

    # number of snps == number of rows the input file, minus those filtered by MAF
    expected_number_snps = 936

    # number of chromosomes == number of columns in input file
    expected_number_chromosomes = 120


def test_CommandLineCosiToBin():
    "Try to call cosi_to_binary.py, and check the first output line."
    import subprocess 
    output = subprocess.check_output("python src/cosi_to_binary.py -m 0.05 -f test/test_data/simulations/cosiout_short.hap-1".split())
    expected_firstline = "neutral_sim hap0_pop1 pop1 EUR X 1 1 1 1 1 1 1 1 1 1 1 1 1"
    firstline = output.split('\n')[2]
#    nose.tools.assert_sequence_equal(firstline, expected_firstline)
    nose.tools.assert_equal(firstline, expected_firstline)
       
def test_CommandLineCosiToBinMafFilter010():
    "Try to call cosi_to_binary.py, using a MAF filter 0.10, and check the first output line."
    import subprocess 
    output = subprocess.check_output("python src/cosi_to_binary.py -f test/test_data/simulations/cosiout_short.hap-1 -m 0.10".split())
    expected_firstline = "neutral_sim hap0_pop1 pop1 EUR X 1 1 1 1 1 1"
    firstline = output.split('\n')[2]
#    nose.tools.assert_sequence_equal(firstline, expected_firstline)
    nose.tools.assert_equals(firstline, expected_firstline)
    
if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
import subprocess
def testDebugReport():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output #doctest: +NORMALIZE_WHITESPACE 
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 3 5 2.0 window_0 16
    ALG3 16 1700 global_noAMR 5 5 1.8 window_0 16
    ALG3 16 370 AFR 4 4 1.7333 window_0 16
    ALG3 16 572 ASN 2 2 1.3333 window_0 16
    ALG3 16 758 EUR 3 3 1.3333 window_0 16
    ALG10B 67 2184 glob 22 10 1.4366 window_0 67
    ALG10B 67 1700 global_noAMR 22 8 1.3115 window_0 67
    ALG10B 67 370 AFR 18 4 0.7143 window_0 67
    ALG10B 67 572 ASN 9 3 1.0588 window_0 67
    ALG10B 67 758 EUR 10 7 1.3548 window_0 67
    CCR5 31 2184 glob 9 7 1.92 window_0 31
    CCR5 31 1700 global_noAMR 7 7 1.907 window_0 31
    CCR5 31 370 AFR 5 4 1.6 window_0 31
    CCR5 31 572 ASN 4 5 1.6667 window_0 31
    CCR5 31 758 EUR 5 5 1.8095 window_0 31
    <BLANKLINE>

  
    """
def testDebugHDF5Report():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data --hdf5"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output #doctest: +NORMALIZE_WHITESPACE 
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 3 5 2.0 window_0 16
    ALG3 16 1700 global_noAMR 5 5 1.8 window_0 16
    ALG3 16 370 AFR 4 4 1.7333 window_0 16
    ALG3 16 572 ASN 2 2 1.3333 window_0 16
    ALG3 16 758 EUR 3 3 1.3333 window_0 16
    ALG10B 67 2184 glob 22 10 1.4366 window_0 67
    ALG10B 67 1700 global_noAMR 22 8 1.3115 window_0 67
    ALG10B 67 370 AFR 18 4 0.7143 window_0 67
    ALG10B 67 572 ASN 9 3 1.0588 window_0 67
    ALG10B 67 758 EUR 10 7 1.3548 window_0 67
    CCR5 31 2184 glob 9 7 1.92 window_0 31
    CCR5 31 1700 global_noAMR 7 7 1.907 window_0 31
    CCR5 31 370 AFR 5 4 1.6 window_0 31
    CCR5 31 572 ASN 4 5 1.6667 window_0 31
    CCR5 31 758 EUR 5 5 1.8095 window_0 31
    <BLANKLINE>

    """
def testDebugReportMedianVar():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug_median.yaml --no-debug --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> print output #doctest: +NORMALIZE_WHITESPACE 
    gene pop max_degree av_degree median_degree median_closeness max_closeness median_betweenness max_betweenness window whole_gene_nsnps
    ALG3 glob 5 2.0 2.0 0.1024 0.1141 5.0667 45.0333 window_0 16
    ALG3 global_noAMR 5 1.8 1.0 0.1067 0.1166 0.0 30.8333 window_0 16
    ALG3 AFR 4 1.7333 1.0 0.1239 0.1359 0.0 11.6667 window_0 16
    ALG3 ASN 2 1.3333 1.0 0.2381 0.25 0.0 1.0 window_0 16
    ALG3 EUR 3 1.3333 1.0 0.16 0.1667 0.0 3.0 window_0 16
    ALG10B glob 10 1.4366 1.0 0.0156 0.0164 0.0 45.0 window_0 67
    ALG10B global_noAMR 8 1.3115 1.0 0.0178 0.0189 0.0 28.0 window_0 67
    ALG10B AFR 4 0.7143 1.0 0.037 0.0434 0.0 9.0 window_0 67
    ALG10B ASN 3 1.0588 1.0 0.0708 0.0766 0.0 3.5 window_0 67
    ALG10B EUR 7 1.3548 1.0 0.0409 0.0417 0.0 21.0 window_0 67
    CCR5 glob 7 1.92 1.0 0.0267 0.0293 0.0 75.3333 window_0 31
    CCR5 global_noAMR 7 1.907 1.0 0.0309 0.0355 0.0 71.3 window_0 31
    CCR5 AFR 4 1.6 1.0 0.0586 0.076 0.0 15.0 window_0 31
    CCR5 ASN 5 1.6667 1.0 0.0819 0.0899 0.0 18.0 window_0 31
    CCR5 EUR 5 1.8095 2.0 0.0658 0.0707 0.5 12.5 window_0 31
    <BLANKLINE>

      
    """


def testDebugReportDistance2():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --distance 2 --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output #doctest: +NORMALIZE_WHITESPACE
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 2 9 5.2727 window_0 16
    ALG3 16 1700 global_noAMR 2 9 5.0 window_0 16
    ALG3 16 370 AFR 1 8 4.2667 window_0 16
    ALG3 16 572 ASN 1 3 2.6667 window_0 16
    ALG3 16 758 EUR 2 5 3.1111 window_0 16
    ALG10B 67 2184 glob 15 10 4.6479 window_0 67
    ALG10B 67 1700 global_noAMR 15 8 3.6721 window_0 67
    ALG10B 67 370 AFR 14 5 1.5714 window_0 67
    ALG10B 67 572 ASN 7 4 2.1176 window_0 67
    ALG10B 67 758 EUR 8 7 3.7419 window_0 67
    CCR5 31 2184 glob 6 17 5.88 window_0 31
    CCR5 31 1700 global_noAMR 5 16 5.7674 window_0 31
    CCR5 31 370 AFR 4 7 3.2 window_0 31
    CCR5 31 572 ASN 3 8 4.2222 window_0 31
    CCR5 31 758 EUR 4 7 4.2857 window_0 31
    <BLANKLINE>

    """

def testDebugReportReadGraphml():
    """
    Test whether the ReportFile can correctly read graphml files.

    Note: I need to save the graphml files before running this test, be careful when debugging.

    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --write_graphml --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --read_graphml_if_possible --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output #doctest: +NORMALIZE_WHITESPACE
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 3 5 2.0 window_0 16
    ALG3 16 1700 global_noAMR 5 5 1.8 window_0 16
    ALG3 16 370 AFR 4 4 1.7333 window_0 16
    ALG3 16 572 ASN 2 2 1.3333 window_0 16
    ALG3 16 758 EUR 3 3 1.3333 window_0 16
    ALG10B 67 2184 glob 22 10 1.4366 window_0 67
    ALG10B 67 1700 global_noAMR 22 8 1.3115 window_0 67
    ALG10B 67 370 AFR 18 4 0.7143 window_0 67
    ALG10B 67 572 ASN 9 3 1.0588 window_0 67
    ALG10B 67 758 EUR 10 7 1.3548 window_0 67
    CCR5 31 2184 glob 9 7 1.92 window_0 31
    CCR5 31 1700 global_noAMR 7 7 1.907 window_0 31
    CCR5 31 370 AFR 5 4 1.6 window_0 31
    CCR5 31 572 ASN 4 5 1.6667 window_0 31
    CCR5 31 758 EUR 5 5 1.8095 window_0 31
    <BLANKLINE>

     
    """

def testDebugReportSavePickle():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --write_pickle --overwrite_existing --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output #doctest: +NORMALIZE_WHITESPACE
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 3 5 2.0 window_0 16
    ALG3 16 1700 global_noAMR 5 5 1.8 window_0 16
    ALG3 16 370 AFR 4 4 1.7333 window_0 16
    ALG3 16 572 ASN 2 2 1.3333 window_0 16
    ALG3 16 758 EUR 3 3 1.3333 window_0 16
    ALG10B 67 2184 glob 22 10 1.4366 window_0 67
    ALG10B 67 1700 global_noAMR 22 8 1.3115 window_0 67
    ALG10B 67 370 AFR 18 4 0.7143 window_0 67
    ALG10B 67 572 ASN 9 3 1.0588 window_0 67
    ALG10B 67 758 EUR 10 7 1.3548 window_0 67
    CCR5 31 2184 glob 9 7 1.92 window_0 31
    CCR5 31 1700 global_noAMR 7 7 1.907 window_0 31
    CCR5 31 370 AFR 5 4 1.6 window_0 31
    CCR5 31 572 ASN 4 5 1.6667 window_0 31
    CCR5 31 758 EUR 5 5 1.8095 window_0 31
    <BLANKLINE>

      
    >>> print subprocess.check_output(['ls', 'test/test_data/pickles/ALG3_glob_distance1.pickle'])
    test/test_data/pickles/ALG3_glob_distance1.pickle
    <BLANKLINE>

    """

def testDebugReportSaveGraphML():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --write_graphml --overwrite_existing --data_basedir=./test/test_data"
    >>> import logging
    >>> logging.debug(command)
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output #doctest: +NORMALIZE_WHITESPACE
    gene n_snps n_datapoints pop n_components max_degree av_degree window whole_gene_nsnps
    ALG3 16 2184 glob 3 5 2.0 window_0 16
    ALG3 16 1700 global_noAMR 5 5 1.8 window_0 16
    ALG3 16 370 AFR 4 4 1.7333 window_0 16
    ALG3 16 572 ASN 2 2 1.3333 window_0 16
    ALG3 16 758 EUR 3 3 1.3333 window_0 16
    ALG10B 67 2184 glob 22 10 1.4366 window_0 67
    ALG10B 67 1700 global_noAMR 22 8 1.3115 window_0 67
    ALG10B 67 370 AFR 18 4 0.7143 window_0 67
    ALG10B 67 572 ASN 9 3 1.0588 window_0 67
    ALG10B 67 758 EUR 10 7 1.3548 window_0 67
    CCR5 31 2184 glob 9 7 1.92 window_0 31
    CCR5 31 1700 global_noAMR 7 7 1.907 window_0 31
    CCR5 31 370 AFR 5 4 1.6 window_0 31
    CCR5 31 572 ASN 4 5 1.6667 window_0 31
    CCR5 31 758 EUR 5 5 1.8095 window_0 31
    <BLANKLINE>

      
    >>> print subprocess.check_output(['ls', 'test/test_data/graphml/ALG3_glob_distance1.graphmlz'])
    test/test_data/graphml/ALG3_glob_distance1.graphmlz
    <BLANKLINE>

    """

def testFullReport():
    """
    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p params/default.yaml --no-debug --data_basedir=./test/test_data"
    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)
    >>> print output #doctest: +NORMALIZE_WHITESPACE
    gene continent distance_definition n_snps central_snp chromosome central_snp_position upstream_position downstream_position region_size distance_from_upstream_margin distance_from_downstream_margin n_datapoints av_datapoints_per_node n_vertices n_edges n_components av_path_length var_path_length diameter av_w_path_length av_w_path_length_inv av_degree var_degree median_degree max_degree av_w_degree av_w_degree_inv av_closeness var_closeness median_closeness max_closeness density av_betweenness var_betweenness median_betweenness max_betweenness window whole_gene_nsnps
    ALG3 glob 1 16 NA NA nan nan nan nan nan nan 2184 99.2727 22 22 3 3.0167 2.5207 7 0.5917 217.6912 2.0 1.4545 2.0 5 0.1957 1009.8819 0.0903 0.0006 0.1024 0.1141 0.0952 11.0 189.4226 5.0667 45.0333 window_0 16
    ALG3 global_noAMR 1 16 NA NA nan nan nan nan nan nan 1700 85.0 20 18 5 2.5976 1.6262 5 0.686 98.7069 1.8 1.76 1.0 5 0.2115 522.6159 0.0905 0.0008 0.1067 0.1166 0.0947 6.55 101.0368 0.0 30.8333 window_0 16
    ALG3 AFR 1 16 NA NA nan nan nan nan nan nan 370 24.6667 15 13 4 2.125 1.0353 4 0.2759 29.5148 1.7333 1.1289 1.0 4 0.111 189.3458 0.1073 0.0008 0.1239 0.1359 0.1238 3.0 17.5667 0.0 11.6667 window_0 16
    ALG3 ASN 1 16 NA NA nan nan nan nan nan nan 572 95.3333 6 4 2 1.3333 0.2667 2 0.5676 7.8253 1.3333 0.2222 1.0 2 0.2838 121.1376 0.2421 0.0 0.2381 0.25 0.2667 0.3333 0.2222 0.0 1.0 window_0 16
    ALG3 EUR 1 16 NA NA nan nan nan nan nan nan 758 84.2222 9 6 3 1.5833 0.447 3 0.6569 11.6248 1.3333 0.6667 1.0 3 0.2838 197.9591 0.1553 0.0003 0.16 0.1667 0.1667 0.7778 1.284 0.0 3.0 window_0 16
    ALG10B glob 1 67 NA NA nan nan nan nan nan nan 2184 30.7606 71 51 22 2.0386 0.6101 4 0.2408 99.9961 1.4366 2.4432 1.0 10 0.0698 772.012 0.0154 0.0 0.0156 0.0164 0.0205 3.0282 63.0696 0.0 45.0 window_0 67
    ALG10B global_noAMR 1 67 NA NA nan nan nan nan nan nan 1700 27.8689 61 40 22 1.9542 0.6594 4 0.2419 95.2557 1.3115 1.7227 1.0 8 0.067 570.323 0.0177 0.0 0.0178 0.0189 0.0219 2.0492 25.2107 0.0 28.0 window_0 67
    ALG10B AFR 1 67 NA NA nan nan nan nan nan nan 370 13.2143 28 10 18 1.6667 0.5333 3 0.1846 17.1719 0.7143 0.7755 1.0 4 0.0396 39.6161 0.0379 0.0 0.037 0.0434 0.0265 0.5 3.25 0.0 9.0 window_0 67
    ALG10B ASN 1 67 NA NA nan nan nan nan nan nan 572 33.6471 17 9 9 1.5294 0.3897 3 0.3632 11.3932 1.0588 0.9965 1.0 3 0.1238 216.106 0.0672 0.0001 0.0708 0.0766 0.0662 0.5294 1.1021 0.0 3.5 window_0 67
    ALG10B EUR 1 67 NA NA nan nan nan nan nan nan 758 24.4516 31 21 10 1.9104 0.6282 4 0.5247 48.6051 1.3548 1.7128 1.0 7 0.1499 300.2533 0.0377 0.0 0.0409 0.0417 0.0452 1.9677 25.3861 0.0 21.0 window_0 67
    CCR5 glob 1 31 NA NA nan nan nan nan nan nan 2184 43.68 50 48 9 2.4134 0.9549 5 0.296 182.6541 1.92 2.4336 1.0 7 0.1032 1036.5362 0.0254 0.0 0.0267 0.0293 0.0392 7.18 247.4076 0.0 75.3333 window_0 31
    CCR5 global_noAMR 1 31 NA NA nan nan nan nan nan nan 1700 39.5349 43 41 7 2.3561 0.9069 5 0.3412 143.8398 1.907 2.2704 1.0 7 0.1171 861.2989 0.0303 0.0 0.0309 0.0355 0.0454 6.4651 218.5459 0.0 71.3 window_0 31
    CCR5 AFR 1 31 NA NA nan nan nan nan nan nan 370 18.5 20 16 5 1.9524 0.8269 4 0.2458 25.882 1.6 0.64 1.0 4 0.0991 146.8105 0.0637 0.0001 0.0586 0.076 0.0842 2.0 13.3 0.0 15.0 window_0 31
    CCR5 ASN 1 31 NA NA nan nan nan nan nan nan 572 31.7778 18 15 4 2.0 0.6531 4 0.5266 46.2719 1.6667 1.4444 1.0 5 0.1909 336.7237 0.08 0.0001 0.0819 0.0899 0.098 2.7778 26.2006 0.0 18.0 window_0 31
    CCR5 EUR 1 31 NA NA nan nan nan nan nan nan 758 36.0952 21 19 5 1.8727 0.5946 4 0.3266 14.0487 1.8095 1.4875 2.0 5 0.1626 439.5752 0.0635 0.0001 0.0658 0.0707 0.0905 2.2857 13.384 0.5 12.5 window_0 31
    <BLANKLINE>

"""



#!/usr/bin/env python
"""
This file contains exceptions raised by the VCF2Networks project
"""
import os
class InvalidIndividualsFile(Exception):
    def __init__(self, message, individuals_file):
        print 
        print "ERROR: The Individuals file provided with the --individuals option is invalid, or missing."
        print "\nMore details on the error:                \n\t" + message + '\n'
        print "This is the individuals_file that you provided with the --individuals option:\n\t'" + individuals_file + "'"
        print 
        print "Please use vcf2networks --help to get usage information"
        print
#        print "\n\nPlease use vcf2networks --help to get usage information"
        print """\nThis is an example of how the individuals_file should look like:

    #ID         POP     CONTINENT       PHENOTYPE1
    HG00096	GBR	EUR	        pink
    HG00097	GBR	EUR	        blue
    HG00099	GBR	EUR	        blue
    HG00100	GBR	EUR	        pink
    HG00101	GBR	EUR	        blue
    HG00102	GBR	EUR	        blue
    HG00103	GBR	EUR	        pink
    HG00104	GBR	EUR	        blue
    HG00106	GBR	EUR	        blue
    HG00108	GBR	EUR	        pink
    ...........
    
    """

        if os.path.isfile(individuals_file): 
            print "These are the first lines of your individuals_file, {0}:".format(individuals_file)
            indfile = open(individuals_file, 'r')
            for x in range(11):
                print '    ' + indfile.readline(), 
#            for line in firstlines:
#                print "\t" + line,

        print "\nCheck that your individuals_file respects the correct format, and that all individuals are defined."
        print "\nCheck also our tutorial at https://bitbucket.org/dalloliogm/vcf2networks for other examples.\n\n\n"
        os.sys.exit(1)

class InvalidPhenotype(Exception):
    def __init__(self, message, individuals_file, phenotype):
        print 
        print "ERROR: The phenotype specified does not exists."
        print "\nMore details on the error:                \n\t" + message + '\n'
        print "This is the individuals_file that you provided with the --individuals option:\n\t'" + individuals_file + "'"
        print "This is the phenotype you requested:\n\t'" + phenotype + "'"
        print 
        print "Please use vcf2networks --help to get usage information"
        print
        if os.path.isfile(individuals_file): 
            print "These are the first lines of your individuals_file, {0}. There it should be a column named '{1}'.".format(individuals_file, phenotype)
            indfile = open(individuals_file, 'r')
            for x in range(11):
                print '    ' + indfile.readline(), 
#            for line in firstlines:
#                print "\t" + line,

        print "\nCheck that your individuals_file respects the correct format, and that all individuals are defined."
        print "\nCheck also our tutorial at https://bitbucket.org/dalloliogm/vcf2networks for other examples.\n\n\n"
        os.sys.exit(1)


class InvalidNetworkAttribute(Exception):
    def __init__(self, message, attributes):
#        print 
        filtered_attributes = ('allcontinents', 'allphenotype_statuses')
        for att in filtered_attributes:
            attributes.remove(att)
        self.msg = "\nERROR: One or more of the attributes requested in the config file are not available. They may not be implemented, or simply mispelled."
        self.msg += "\n\nMore details on the error:                \n\t" + message + '\n'
        self.msg += "\n"
        self.msg += "\nHere is a list of all the valid attributes:"
        self.msg += '\n - ' + '\n - '.join(sorted(attributes))
        self.msg += '\n\nA complete description of the attributes is available in the supplementary materials of the paper describing VCF2Networks (unpublished)\n\n'
        print self.msg
        os.sys.exit(1)

#    def __str__(self):
#        print self.msg

class GenotypeNetworkInitError(Exception):
    """error raised when something went wrong with Network initialization,
    such as an improper genotype string or a wrong parameter"""
    def __init__(self, message=""):
        print "ERROR: an error occurred when initializing the Genotype Network. Probably one of the input file is not formatted properly."
        print
        print message

class GenotypeNetworkAttributeError(Exception):
    """error raised when something tried to access to an attribute
    of the genotype network that doesn't exists"""
    def __init__(self, message=""):
        print "ERROR: " + message











if __name__ == '__main__':
    pass




#!/usr/bin/env python
import subprocess
import operator
import random

def testDebugReportIndividualSubsampling():
    """
    Check that the individuals subsampled are always the same.

    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data --random_seed 2 -n 50 -t individuals_line"

    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> output_lines = output.split('\\n')

    >>> for line in output_lines: #doctest: +NORMALIZE_WHITESPACE 
    ...     print line[0:60]
    <BLANKLINE>
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    <BLANKLINE>

    >>> output_lines = output_lines[1:-1]
    >>> output_fields = [line.split() for line in output_lines]

    # Check that all the columns, except for the ones starting with "--", have the same values for all the output lines.
    # This means that all the selected individuals are the same, for each line.

    # Select 500 random columns:
    >>> random_columns = random.sample(xrange(len(output_fields[0])), 500)
    >>> fields_getter = operator.itemgetter(*random_columns)
    >>> ALG3_columns = fields_getter(output_fields[0])
    >>> ALG10B_columns = fields_getter(output_fields[1])
    >>> CCR5_columns = fields_getter(output_fields[2])
    >>> for column_id in xrange(len(ALG3_columns)):
    ...     if not ALG3_columns[column_id].startswith("----"):
    ...        if not ALG3_columns[column_id] == ALG10B_columns[column_id] == CCR5_columns[column_id]:
    ...             print "not matching values", ALG3_columns[column_id], ALG10B_columns[column_id], CCR5_columns[column_id]
    """

def testDebugReportIndividualSubsamplingReplicates():
    """
    Check that the individuals subsampled are always the same.

    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data --random_seed 2 -n 50 -t individuals_line -s 5"

    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> output_lines = output.split('\\n')

    >>> for line in output_lines: #doctest: +NORMALIZE_WHITESPACE 
    ...     print line[0:60]
   <BLANKLINE>
    -----ALG3 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----ALG3 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----ALG3 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----ALG3 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----ALG3 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    <BLANKLINE>

    >>> output_lines = output_lines[1:-1]
    >>> output_fields = [line.split() for line in output_lines]


    # Check that all the columns, except for the ones starting with "--", have the same values for all the output lines.
    # This means that all the selected individuals are the same, for each line.
    >>> random_columns = random.sample(xrange(len(output_fields[0])), 500)
    >>> fields_getter = operator.itemgetter(*random_columns)
    >>> for replica_id in range(5):
    ...     ALG3_columns = fields_getter(output_fields[replica_id])
    ...     ALG10B_columns = fields_getter(output_fields[replica_id+5])
    ...     CCR5_columns = fields_getter(output_fields[replica_id+10])
    ...     for column_id in xrange(len(ALG3_columns)):
    ...         if not ALG3_columns[column_id].startswith("----"):
    ...             if not ALG3_columns[column_id] == ALG10B_columns[column_id] == CCR5_columns[column_id]:
    ...                 print "not matching values", ALG3_columns[column_id], ALG10B_columns[column_id], CCR5_columns[column_id]

  
    """

def testDebugReportIndividualSubsamplingMultipleWindows():
    """
    Check that the individuals subsampled are always the same.

    >>> command = "python src/generate_network_report.py  -f test/test_data/params/pathways/debug_pathway.genes -p test/test_data/params/debug.yaml --no-debug --data_basedir=./test/test_data --random_seed 2 -n 50 -t individuals_line -w 20"

    >>> output = subprocess.check_output(command.split(), stderr=subprocess.PIPE)

    #NOTE: the blankline at the end is generated by subprocess.check_output, not by the script
    >>> output_lines = output.split('\\n')

    >>> for line in output_lines: #doctest: +NORMALIZE_WHITESPACE 
    ...     print line[0:60]
    <BLANKLINE>
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----ALG10B AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA184
    -----CCR5 AFR  NA18486_1 AFR YRI  NA18487_1 AFR YRI  NA18489
    <BLANKLINE>

    >>> output_lines = output_lines[1:-1]
    >>> output_fields = [line.split() for line in output_lines]

    # Check that all the columns, except for the ones starting with "--", have the same values for all the output lines.
    # This means that all the selected individuals are the same, for each line.

    # Select 500 random columns:
    >>> random_columns = random.sample(xrange(len(output_fields[0])), 500)
    >>> fields_getter = operator.itemgetter(*random_columns)
    >>> ALG10B_w1_columns = fields_getter(output_fields[0])
    >>> ALG10B_w2_columns = fields_getter(output_fields[1])
    >>> ALG10B_w3_columns = fields_getter(output_fields[2])
    >>> CCR5_columns = fields_getter(output_fields[3])
    >>> for column_id in xrange(len(ALG10B_w1_columns)):
    ...     if not ALG10B_w1_columns[column_id].startswith("----"):
    ...        if not ALG10B_w1_columns[column_id] == ALG10B_w2_columns[column_id] == ALG10B_w3_columns[column_id] == CCR5_columns[column_id]:
    ...             print "not matching values", ALG10B_w1_columns[column_id], ALG10B_w2_columns[column_id], ALG10B_w3_columns[column_id], CCR5_columns[column_id]
    """



if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Convert a COSI .hap output to a binary file


Example .binary file:

    ALG10 HG00096_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00097_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00099_1 GBR EUR X 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0
    ALG10 HG00100_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00101_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 0 1 1 1 0 0 0 0 0 0 1 0 0 0
    ALG10 HG00102_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00103_1 GBR EUR X 1 1 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0
    ALG10 HG00104_1 GBR EUR X 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 1 0 1 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00106_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00108_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    ALG10 HG00109_1 GBR EUR X 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0

"""
import optparse
import logging
import os
import numpy
import gzip

def get_options():
    parser = optparse.OptionParser(usage="python %prog -f hapfile", description=__doc__)
    parser.add_option('-f', '--hapfile', dest='hapfile', 
            help='Cosi .hap file', default=False)
    parser.add_option('-u', '--no-debug', dest='debug', action='store_false', default=True)
    parser.add_option('-m', '--maf', '--maf-filter', dest='maf_filter', type='float',
            help='Maf Filter. Must be a float number, from 0 to 1. Default: %default',
            default=0.01)
    (options, args) = parser.parse_args()
    
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, filename='logs/cosi_to_binary.log')

    if (options.hapfile is False) and (len(args) == 0):
        parser.print_help()
        parser.error('cosi_to_binary.py error: no cosi hap file provided')

    try:
        hapfile_path = ''
        if options.hapfile is not False:
            hapfile_path = options.hapfile
        elif args != '':
            hapfile_path = args[0]
        if os.stat(hapfile_path).st_size == 0:
            parser.error("file {0} is empty".format(hapfile_path))
    except:
        print __doc__
        parser.error("Can not open cosi hap file {0}".format(options.hapfile))


    return (hapfile_path, options.maf_filter)



def read_pos_file(posfile_path, maf_filter=0.01):
    """
    Read a COSI pos file and get a list of all the SNPs that pass a MAF filter.

    NOTE: this function is implemented, but not used in the analysis, as I prefer to make the MAF filtering step when reading the hap file.


    Example pos_file:

    >>> from tempfile import NamedTemporaryFile
    >>> posfile = NamedTemporaryFile(delete=False)
    >>> posfile.write(r'''
    ... SNP     CHROM   CHROM_POS       ALLELE1 FREQ1   ALLELE2 FREQ2
    ... 1       1       36      1       0.0000  2       1.0000
    ... 2       1       304     1       0.5000  2       0.5000
    ... 3       1       323     1       0.0001  2       0.9999
    ... 4       1       471     1       0.0000  2       1.0000
    ... 5       1       561     1       0.0000  2       1.0000
    ... 6       1       683     1       0.6000  2       0.4000
    ... 7       1       856     1       0.0000  2       1.0000
    ... 8       1       972     1       0.0167  2       0.9833
    ... 9       1       1312    1       1.0000  2       0.0000
    ... ''')
    >>> posfile.close()


    After filtering, SNPs 1, 3, 4, 5, 7 and 9 will be filtered out, because they don't pass the MAF filter test.
    >>> filtered_snps = read_pos_file(posfile.name)
    >>> print filtered_snps
    ['2', '6', '8']

    >>> posfile.unlink(posfile.name)
    """
    snps = []
    with open(posfile_path, 'r') as posfile_h:
        for line in posfile_h:
            if not line.strip() == '':
                if not line[0] in ('S', '#'):
                    fields = line.split()
                    snp_id = fields[0]
                    freq1 = float(fields[4])
                    if (freq1 < maf_filter) or (1-freq1 < maf_filter):
#                        print "filtered out", snp_id, freq1
                        pass
                    else:
                        snps.append(snp_id)
    return snps

sim_pops_dict = {'1': 'EUR', 
        '2': 'ASN',
        '3': 'AFR'
        }

def read_hap_file(cosifile_path, maf_filter=0.01):
    """
    Read a cosi .HAP file, containing the haplotypes of the individuals.

    maf_filter is a minor allele frequency filter. All the SNPs that have a maf lower than it, will be removed.

    =======================
    Example Input .hap File
    =======================

    Explanation of input cosi .hap file format: 

     * first column is the individual id
     * second column is the population ID
     * other columns are genotypes, one column per SNP.

    Each line is a different chromosome, one of the two haplotypes of each individual.

    Note that in this example, the first two columns are monomorphic (they are all 2s or 1s) and according to the MAF filter they will be removed.

    >>> from tempfile import NamedTemporaryFile
    >>> hapfile = NamedTemporaryFile(prefix='hap1', delete=False)
    >>> hapfile.write(r'''0       1       2 1 2 2 2 
    ... 1       1       2 1 2 2 2
    ... 2       1       2 1 2 2 1
    ... 3       1       2 1 2 2 2
    ... 4       1       2 1 1 1 1
    ... 5       1       2 1 1 2 2
    ... ''')
    >>> hapfile.close()

    ===========================
    Example Output binary file
    ===========================

    Explanation of output .binary file format: 

     * first column is a label to indicate that these are neutral simulations
     * second column is an id for population of the individual.
     * other columns are binary genotypes, one column per SNP.

    Each line is a different chromosome. The individuals id don't really matter, as these are fake individuals generated by the simulations.

    >>> print (read_hap_file(hapfile.name))  #doctest: +NORMALIZE_WHITESPACE
    #binary file v2. Simulations results. MAF filter: 0.01. Only SNPs that have MAF > 0.01 in each population are kept.
    #number of SNPs: 5. SNPs filtered: 2
    neutral_sim hap0_pop1 pop1 EUR X 1 1 1
    neutral_sim hap1_pop1 pop1 EUR X 1 1 1
    neutral_sim hap2_pop1 pop1 EUR X 1 1 0
    neutral_sim hap3_pop1 pop1 EUR X 1 1 1
    neutral_sim hap4_pop1 pop1 EUR X 0 0 0
    neutral_sim hap5_pop1 pop1 EUR X 0 1 1
    <BLANKLINE>

    >>> hapfile.unlink(hapfile.name)
    """
#    if pop_label is None:
#        pop_label = cosifile_path.split('.')[-1][-1]
    print "#binary file v2. Simulations results. MAF filter: {0}. Only SNPs that have MAF > {0} in each population are kept.".format(maf_filter)
    outputtemplate = "neutral_sim\tpop{0}\t{1}\n"
    output = ''
    if cosifile_path.split('.')[-1] == 'gz':
        cosifile = gzip.open(cosifile_path)
    else:
        cosifile = open(cosifile_path, 'r')

    firstline = ""
    while firstline is "":
        firstline = cosifile.readline().strip()
##            print firstline
    fields = firstline.split()
    first_haplotype = [int(g)-1 for g in fields[2:]]
    first_haplotype_pop = fields[1]
    n_snps = len(first_haplotype)
#   print n_snps
    cosifile.seek(0)

    all_haplotypes_bypop = {}
    # The first line is first added and then removed... not very nice, but effective.
    all_haplotypes_bypop['1'] = numpy.matrix(first_haplotype)
    all_haplotypes_bypop['2'] = numpy.matrix(first_haplotype)
    all_haplotypes_bypop['3'] = numpy.matrix(first_haplotype)
    for line in cosifile:
        logging.debug( len(line))
        logging.debug( line)
        if len(line)>1:
            fields = line.split()
#        ind_id = fields[0]
            pop_id = fields[1]
            pop = sim_pops_dict[pop_id]
            current_haplotype = [int(g)-1 for g in fields[2:]]
            # Append current chromosome to matrix. 
            # TODO: see if there is a better way to append a list to a matrix, in place
            all_haplotypes_bypop[pop_id] = numpy.vstack((all_haplotypes_bypop[pop_id], current_haplotype))
    all_haplotypes_bypop['1'] = numpy.delete(all_haplotypes_bypop['1'], (0), axis=0)
    all_haplotypes_bypop['2'] = numpy.delete(all_haplotypes_bypop['2'], (0), axis=0)
    all_haplotypes_bypop['3'] = numpy.delete(all_haplotypes_bypop['3'], (0), axis=0)
    logging.debug(all_haplotypes_bypop)
#    print all_haplotypes_bypop

    # Apply maf filter 
    filtered_columns = []       
    for (popid, all_haplotypes) in all_haplotypes_bypop.items():
#        print popid, len(all_haplotypes)
        if len(all_haplotypes) == 1:
            continue
        for position in xrange(n_snps):
            current_freq = numpy.average(all_haplotypes[:, position])
            if (1-current_freq < maf_filter) or (current_freq < maf_filter):
                filtered_columns.append(position)

#    print filtered_columns
    print "#number of SNPs: {0}. SNPs filtered: {1}".format(n_snps, len(set(filtered_columns)))
    for popid, all_haplotypes in all_haplotypes_bypop.items():
        indlabel = "hap{0}_pop" + popid
        poplabel = "pop" + popid
        contlabel = sim_pops_dict[popid]
        kept_columns = [n for n in xrange(n_snps) if n not in filtered_columns]
        all_haplotypes = all_haplotypes[:, kept_columns]
        all_haplotypes_list = all_haplotypes.tolist()
        for hap_id in range(len(all_haplotypes_list)):
            chromosome = all_haplotypes_list[hap_id]
            output += "neutral_sim " + indlabel.format(hap_id) + " " + poplabel + " " + contlabel + " X " + ' '.join(str(h) for h in chromosome) + "\n"
    return output

def main():
    (hapfile, maf_filter) = get_options()
    print read_hap_file(hapfile, maf_filter)

if __name__ == '__main__':
    main()





#!/usr/bin/env python
#
# setup for the VCF2Space suite
#
# use the following to install:
# 
# $: python setup.py install
#

import os
from setuptools import setup
from os.path import join, dirname
import src

scripts = [os.path.join('src', x) for x in os.listdir('src')]
long_description = """VCF2Networks is a tool to apply Genotype Networks to Single Nucleotide Polymorphism data.

Read more at https://bitbucket.org/dalloliogm/vcf2networks 


Note on installation: this module depends on the python-igraph library, which requires a C igraph library which must be compiled separately. See https://pypi.python.org/pypi/python-igraph for more instructions. See also https://bitbucket.org/dalloliogm/vcf2networks/src/tip/docs/installing.rst for our tips.
"""

setup(name = 'vcf2networks',
    version = src.__version__,
    description = 'calculate Genotype Networks from VCF files',
    long_description = long_description,
#    long_description = open('README.rst').read(),
    author = "Giovanni M. Dall'Olio",
    author_email = "giovanni.dallolio@upf.edu",
    url = 'https://bitbucket.org/dalloliogm/vcf2networks',
    download_url = 'https://bitbucket.org/dalloliogm/vcf2networks/get/tip.zip',
    packages=['src'],
#    scripts = scripts,
    classifiers=[
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics"
    ],
    keywords='genetics, variant, nucleotide variant, vcf, population genetics, bioinformatics, networks, genotype networks',
    license='GPL',

    entry_points={
        'console_scripts':
            ['vcf2networks = src.vcf2networks:main']
        },

#    data_files = ['README.rst'],
    install_requires=[
#        'setuptools',
        'argparse',
        'python-igraph',
        'numpy',
        'PyYAML'
        ],
)






#!/usr/bin/env python
"""
Parse all simulations, and get a file with distributions of values.
"""
import glob
import binary_to_network 
import yaml
import gzip
import re
import optparse
import random
from GenotypeNetwork import GenotypeNetworkInitError
import numpy
import warnings
import StringIO
import os

def get_options():
    parser = optparse.OptionParser(
			usage="python %prog -f genes_file.genes", 
            description=__doc__)
    config_group = optparse.OptionGroup(parser, 
			"Configuration options. Required")
    config_group.add_option(
			'-f', '-s', '--simulations_folder', dest='simulations_folder', 
            help='File containing the list of genes to be analyzed. Required. Default: %default', 
			default='./data/simulations/binary/globalpops')
    config_group.add_option('-p', '--params', dest='params_file', 
            help='File containing the parameters of the analysis. In particular, the fields to be included in the report. Required. Default: %default', 
            default="params/default.yaml")
    config_group.add_option('--background_type', '--report_type', '-b', '-t', 
			dest='report_type', 
            help='Type of report to produce.\nPossible values: "interpop_comparison", "values_by_gene", "values_by_node", "individuals", and "individuals_line". Default: %default',
            choices=("values_by_gene", "values_by_node", "interpop_comparison",
					"individuals", "individuals_line"),
            default="values_by_gene")
    parser.add_option_group(config_group)
    network_definition_group = optparse.OptionGroup(parser, 
			"Network Definition options")
    network_definition_group.add_option('-n', '-d', '--distance', 
			dest='distance', type='int',
            help='Distance between nodes in the network. Two nodes will be considered being neighbors if they have at least _distance_ number of differences. Default: %defaul',
            default=1)
    network_definition_group.add_option('-w', '--windows_size', '--windows', 
			dest='windows_size', type='int',
            help='Size of window to use. If 0, all data will be included. Default: %default', 
            default=11)
    network_definition_group.add_option('-i', '--individuals', '--number_individuals', 
			dest='n_individuals', type='int',
            help='Number of individuals to be included. If 0, all the individuals are used. Default: %default', 
            default=0)
    network_definition_group.add_option('--random_seed', dest='random_seed', type='int',
            help='seed for random sampling when --individuals is defined. If 0, then a random seed is used. Default: %default', 
            default=0)
    network_definition_group.add_option('-r', '--number_subsampling', dest='n_subsampling', type='int',
            help='How many times to resample individuals, if the -i option is not 0. Default: %default', 
            default=1)
    network_definition_group.add_option('-g', '--global', dest='global_pop', action='store_true', 
            help="Calculate network properties for a global population as well.", default=False)
    parser.add_option_group(network_definition_group)
    notimplemented_group = optparse.OptionGroup(parser, 
			"Options not implemented yet")
    notimplemented_group.add_option('--hdf5', dest='use_hdf5', action='store_true', 
            help='Use HDF5 file instead of binary',
            default=False)
    notimplemented_group.add_option('--parallelize', dest='parallelize', 
			action='store_true',
            help='use multiple processors for generating the report',
            default=False)
    parser.add_option_group(notimplemented_group)
    parser.add_option('-u', '--no-debug', dest='debug', action='store_false', 
            help="disable debugging messages, which are enabled by default", 
			default=True)
    (options, args) = parser.parse_args()
    
    if options.parallelize is True:
        raise NotImplementedError("parallelization wasn't working correctly, so I removed the code for it.")
    if options.report_type in ("interpop_comparison", ):
        raise NotImplementedError("interpop comparison report not implemented yet")
    if options.use_hdf5 is True:
        raise NotImplementedError("no support for HDF5 in simulations yet")
    if options.windows_size == 0:
        raise NotImplementedError("windows_size == 0 not implemented yet")
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, filename='logs/network_report.log')
    
    options.simulations_folder = options.simulations_folder + '/'

    return (options)

class SimulationsParserError(Exception):
    """
    Generic error raise when something goes wrong while parsing sims.
    """
    pass

def get_simulations_background(simulations_folder, 
        windows_size = 11, 
        params_file = 'params/default.yaml', 
        report_type = 'values_by_gene', 
        distance = 1,
        n_individuals = 0,
        n_subsampling = 1,
        random_seed = False):
    """
    Generate a file containing the distribution of values for network 
	 properties in the simulations.

    For each simulation, parse the corresponding file and calculate 
	 many network properties, thanks to the csv_report function.
    Then, print these properties on a new line, and move to the next 
	 simulation.
    """
    continent_map = {
        '1': 'EUR',
        '2': 'ASN',
        '3': 'AFR',
        '0': 'glob',
        'pop1': 'EUR',
        'pop2': 'ASN',
        'pop3': 'AFR',
        'pop0': 'glob'
        }

    if random_seed == 0:
        random_seed = random.random()

    params_file = yaml.load(open(params_file, 'r'))
    report_fields = []
    if report_type == 'values_by_gene':
        report_fields.extend(params_file['networkproperties_report_fields'])
        report_fields.extend(["window", "whole_gene_nsnps"])
    elif report_type == 'values_by_node':
        report_fields.extend(["genotype", "name", "window", "whole_gene_nsnps"])
        report_fields.extend(params_file['allvalues_report_fields'])

    simulations_basedir = simulations_folder

    networks = {}
    header = ' '.join(report_fields) 
    output_background = header
    print header
    options.overlapping = True
    options.use_hdf5 = False

    allsimulations = sorted(glob.glob(simulations_basedir + '*binary.gz'))
    original_random_seed = random_seed

    for simfile_name in allsimulations:
        try:
            
            if os.path.exists(simfile_name) is False:
                break
            simfile = open(simfile_name, 'r')

            # check if file is gzipped
            if simfile.read(2) == "\x1f\x8b":
                simfile = gzip.open(simfile_name, 'r')
            else:
                simfile.seek(0)
            if n_individuals > 0:
                random_seed = original_random_seed
                for subsample in xrange(n_subsampling):
                    random_seed += 1 
#                    print
#                    print random_seed
                    binary_to_network.print_report_and_subgraphs(simfile, 
							"neutral_sim", individuals_dict=False, 
                            pops=["global_sub", "EUR", "ASN", "AFR"], 
							report_type=options.report_type, 
							report_fields=report_fields, 
                            options=options, central_window_only=True,
                            n_individuals=n_individuals, random_seed=random_seed)
            else:
                binary_to_network.print_report_and_subgraphs(simfile, 
						"neutral_sim", individuals_dict=False, 
                        pops=["EUR", "ASN", "AFR"], 
						report_type=options.report_type,
						report_fields=report_fields,
                        options=options, central_window_only=True,
                        n_individuals=n_individuals, random_seed=random_seed)

        except GenotypeNetworkInitError:
            pass
#        print
    return output_background


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    options = get_options()
    get_simulations_background(
                    simulations_folder = options.simulations_folder,
                    windows_size = options.windows_size, 
                    params_file = options.params_file, 
                    report_type = options.report_type, 
                    distance=options.distance,
                    n_individuals=options.n_individuals,
                    n_subsampling=options.n_subsampling,
                    random_seed = options.random_seed)


